/* --------------------------------------------------------------------------
 *
 * Copyright (C) 2007 Leif Erik Larsen, Kjerringvik, Norway.
 *
 * This file is part of the Open Source Edition of Larsen Commander, as
 * available from http://home.online.no/~leifel/lcmd/.  This code is free 
 * software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License version 3 only, as published by the 
 * Free Software Foundation.  
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 3 at http://www.gnu.org/licenses/gpl-3.0.txt for more details 
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * ------------------------------------------------------------------------ */

#include <limits.h>
#include "glib/primitives/GLong.h"

const longlong GLong::MIN_VALUE = (longlong(0xFFFFFFFF) << 32) | longlong(0xFFFFFFFF);
const longlong GLong::MAX_VALUE = (longlong(0x7FFFFFFF) << 32) | longlong(0xFFFFFFFF);

GLong::GLong () 
      :value(0) 
{
}

GLong::GLong ( const GLong& val ) 
      :value(val.value) 
{
}

GLong::GLong ( longlong val ) 
      :value(val) 
{
}

GLong::~GLong ()
{
}

int GLong::hashCode () const
{
   return int(value ^ (value >> 32));
}

bool GLong::equals ( const GObject& obj ) const
{
   if (&obj == this)
      return true;
   const GLong* o = dynamic_cast<const GLong*>(&obj);
   if (o != null)
      return value == o->value;
   return false;
}

longlong GLong::ParseLong ( const GString& str, int radix )
{
   return GIntegerNumber<longlong>::ParseSigned(str, MIN_VALUE, MAX_VALUE, radix);
}

GString GLong::toString () const
{
   return GIntegerNumber<longlong>::ToSignedString(value);
}

GString GLong::ToString ( longlong val, int radix )
{
   return GIntegerNumber<longlong>::ToSignedString(val, radix);
}
