/* --------------------------------------------------------------------------
 *
 * Copyright (C) 2007 Leif Erik Larsen, Kjerringvik, Norway.
 *
 * This file is part of the Open Source Edition of Larsen Commander, as
 * available from http://home.online.no/~leifel/lcmd/.  This code is free 
 * software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License version 3 only, as published by the 
 * Free Software Foundation.  
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 3 at http://www.gnu.org/licenses/gpl-3.0.txt for more details 
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * ------------------------------------------------------------------------ */

#ifndef __GLIB_POINT
#define __GLIB_POINT

#include "glib/primitives/GObject.h"

/**
 * Primitive class used to represent a single [x:y] point.
 * 
 * @author  Leif Erik Larsen
 * @since   2001.03.07
 */
class GPoint : public GObject
{
   public:

      int x;
      int y;

   public:

      GPoint ();
      GPoint ( int x, int y );
      GPoint ( const GPoint& pt );

      virtual ~GPoint ();

   public:

      GPoint& operator= ( const GPoint& pt );
      bool operator== ( const GPoint& pt ) const;

   private:

      GPoint& copyFrom ( const GPoint& pt );

   public:

      void clear ();

      bool equals ( const GPoint& pt ) const;
      bool equals ( int x, int y ) const;

      void set ( int x, int y );

      /**
       * Return a string representation of the point, for debugging
       * purposes only.
       */
      GString toString () const;
};

#endif

