/* --------------------------------------------------------------------------
 *
 * Copyright (C) 2007 Leif Erik Larsen, Kjerringvik, Norway.
 *
 * This file is part of the Open Source Edition of Larsen Commander, as
 * available from http://home.online.no/~leifel/lcmd/.  This code is free 
 * software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License version 3 only, as published by the 
 * Free Software Foundation.  
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 3 at http://www.gnu.org/licenses/gpl-3.0.txt for more details 
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * ------------------------------------------------------------------------ */

#ifndef __GLIB_CHANGEABLEVALUECONTAINER
#define __GLIB_CHANGEABLEVALUECONTAINER

#include "glib/primitives/GString.h"

/**
 * This interface declares methods to set and get some "changeable value"
 * that is implementation dependent. The changeable value has to be a 
 * {@link GString}, though.
 *
 * @author  Leif Erik Larsen
 * @since   2004.05.27
 * @see     GValueChangeListener
 */
class GChangeableValueContainer : public GObject
{
   public:

      /**
       * Change the current value of the object.
       * The meaning of the new value depends on the implementing subclass.
       *
       * @author  Leif Erik Larsen
       * @since   2004.05.27
       * @param   newValue  The new Value of which to change to.
       * @param   notify    True if we shall perform synchronous callback 
       *                    notification to any registered 
       *                    {@link GValueChangeListener}'s, or else false. 
       *                    This parameter should normally be true if it 
       *                    isn't any known reason for setting it false.
       */
      virtual void changeValue ( const GString& newValue, bool notify = true ) = 0;

      /**
       * Get a copy of the current value of the object.
       * The meaning of the returned value depends on the 
       * implementing subclass.
       *
       * @author  Leif Erik Larsen
       * @since   2004.05.27
       * @return  A copy of the current value of the object.
       */
      virtual GString queryValue () const = 0;
};

#endif

